package fr.dgedl.config;

import java.util.Properties;

import javax.sql.DataSource;

import org.apache.commons.lang3.BooleanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.ImportResource;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.dao.annotation.PersistenceExceptionTranslationPostProcessor;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.JpaVendorAdapter;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.Database;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.annotation.EnableTransactionManagement;

@Configuration
@ImportResource({ "classpath:/META-INF/spring/applicationContext-config.xml" })
@ComponentScan({ "fr.dgedl.config", "fr.dgedl.service", "fr.dgedl.repository" })
@PropertySource({ "classpath:/META-INF/dgedl/dgedl.properties" })
@EnableTransactionManagement
public class PersistenceJpaConfiguration {

	private final Log log = LogFactory.getLog(PersistenceJpaConfiguration.class);

	@Autowired
	private Environment env;

	public PersistenceJpaConfiguration() {
		super();
	}

	// beans

	@Bean
	public LocalContainerEntityManagerFactoryBean entityManagerFactoryBean() {
		final LocalContainerEntityManagerFactoryBean factoryBean = new LocalContainerEntityManagerFactoryBean();
		factoryBean.setDataSource(restDataSource());
		factoryBean.setPackagesToScan(new String[] { "fr.dgedl.domain" });

		final JpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter() {
			{
				setDatabase(Database.MYSQL);
				setDatabasePlatform(env.getRequiredProperty("hibernate.dialect"));
				setShowSql(BooleanUtils.toBoolean(env.getRequiredProperty("hibernate.show_sql")));
				setGenerateDdl(BooleanUtils.toBoolean(env.getRequiredProperty("jpa.generateDdl")));
			}
		};

		// use this to inject additional properties in the EntityManager
		final Properties additionalProperties = new Properties();
		additionalProperties.setProperty("hibernate.hbm2ddl.auto", env.getRequiredProperty("hibernate.hbm2ddl.auto"));

		factoryBean.setJpaVendorAdapter(vendorAdapter);
		factoryBean.setJpaProperties(additionalProperties);

		return factoryBean;
	}

	@Bean
	public DataSource restDataSource() {
		if (log.isDebugEnabled()) {
			log.debug("Connecting datasource with properties : ");
			log.debug("jdbc.driverClassName : " + env.getRequiredProperty("jdbc.driverClassName"));
			log.debug("jdbc.url : " + env.getRequiredProperty("jdbc.url"));
		}

		final DriverManagerDataSource dataSource = new DriverManagerDataSource();
		dataSource.setDriverClassName(env.getRequiredProperty("jdbc.driverClassName"));
		dataSource.setUrl(env.getRequiredProperty("jdbc.url"));
		dataSource.setUsername(env.getRequiredProperty("jdbc.username"));
		dataSource.setPassword(env.getRequiredProperty("jdbc.password"));
		return dataSource;
	}

	@Bean
	public JpaTransactionManager transactionManager() {
		final JpaTransactionManager transactionManager = new JpaTransactionManager();
		transactionManager.setEntityManagerFactory(entityManagerFactoryBean().getObject());

		return transactionManager;
	}

	@Bean
	public PersistenceExceptionTranslationPostProcessor persistenceExceptionTranslationPostProcessor() {
		return new PersistenceExceptionTranslationPostProcessor();
	}

}