package fr.dgedl.domain;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import javax.validation.groups.Default;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotEmpty;

import com.fasterxml.jackson.annotation.JsonIgnore;

import fr.dgedl.domain.validation.ContraintsUserCreation;

/**
 * A user.
 * 
 */

@Entity
@Table(name = "User")
public class User implements Serializable {

	private static final long serialVersionUID = 998709605411059982L;

	@Column(name = "login")
	@Id
	@NotEmpty(message = "Login is mandatory.", groups = { ContraintsUserCreation.class, Default.class })
    @NotNull(message = "Login is mandatory.", groups = {ContraintsUserCreation.class, Default.class})
    @Email(message = "Email is invalid.")
    private String login;

    @Column(name = "password")
    @JsonIgnore
    private String password;

    @Column(name = "username")
    private String username;

	public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

	@Override
    public boolean equals(Object o) {
		if (!(o instanceof User)) {
			return false;
		}
		User user = (User) o;

		return new EqualsBuilder().append(login, user.login).isEquals();
    }

    @Override
    public int hashCode() {
		return new HashCodeBuilder(15, 19).append(login).toHashCode();
    }

    @Override
    public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
    }
}