package fr.dgedl.repository.jpa;

import java.io.Serializable;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.springframework.transaction.annotation.Transactional;

@Transactional(readOnly = true)
public class AbstractJpaRepository<T, ID extends Serializable> {

	private Class<T> clazz;

	@PersistenceContext
	protected EntityManager entityManager;

	public void setClazz(Class<T> clazzToSet) {
		this.clazz = clazzToSet;
	}

	public T findOne(ID id) {
		return entityManager.find(clazz, id);
	}

	@SuppressWarnings("unchecked")
	public List<T> findAll() {
		return entityManager.createQuery("from " + clazz.getName()).getResultList();
	}

	@Transactional
	public T save(T entity) {
		entityManager.persist(entity);
		return entity;
	}

	@Transactional
	public T update(T entity) {
		return entityManager.merge(entity);
	}

	@Transactional
	public void delete(T entity) {
		entityManager.remove(entity);
	}

	@Transactional
	public void delete(ID id) {
		final T entity = findOne(id);
		delete(entity);
	}

	@Transactional
	public T saveAndFlush(T entity) {
		T result = save(entity);
		flush();

		return result;
	}

	@Transactional
	public void flush() {
		entityManager.flush();
	}

}