package fr.dgedl.web.init;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletRegistration;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.AnnotationConfigWebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;
import org.springframework.web.servlet.DispatcherServlet;

import fr.dgedl.config.DispatcherServletConfiguration;
import fr.dgedl.config.PersistenceJpaConfiguration;

/**
 * Configuration of web application with Servlet 3.0 APIs.<br>
 * <p/>
 * This class is to be used as a standard listener within a web.xml. To optimise startup time, you can completely disable classpath scanning :
 * <ul>
 * <li>with metadata-complete="true" global attribute
 * <li>with an empty &lt;absolute-ordering&gt; ( it's necessary with Jetty at least <b>TODO</b> : test with other containers )
 * </ul>
 * <p/>
 * See web.xml :
 * <p/>
 * 
 */
public class WebConfigurer implements ServletContextListener {

	private final Log log = LogFactory.getLog(WebConfigurer.class);

	@Override
	public void contextInitialized(ServletContextEvent sce) {
		ServletContext servletContext = sce.getServletContext();
		log.info("Web application configuration");

		log.debug("Configuring Spring root application context");
		AnnotationConfigWebApplicationContext rootContext = new AnnotationConfigWebApplicationContext();
		rootContext.register(PersistenceJpaConfiguration.class);
		rootContext.setServletContext(servletContext);
		rootContext.refresh();

		servletContext.setAttribute(WebApplicationContext.ROOT_WEB_APPLICATION_CONTEXT_ATTRIBUTE, rootContext);

		log.debug("Configuring Spring Web application context");
		AnnotationConfigWebApplicationContext dispatcherServletConfig = new AnnotationConfigWebApplicationContext();
		dispatcherServletConfig.setParent(rootContext);
		dispatcherServletConfig.register(DispatcherServletConfiguration.class);

		log.debug("Registering Spring MVC Servlet");
		ServletRegistration.Dynamic dispatcherServlet = servletContext.addServlet("dispatcher", new DispatcherServlet(dispatcherServletConfig));
		dispatcherServlet.addMapping("/dgedl/*");
		dispatcherServlet.setLoadOnStartup(2);

		log.debug("Web application fully configured");
	}

	@Override
	public void contextDestroyed(ServletContextEvent sce) {
		log.info("Destroying Web application");
		WebApplicationContext ac = WebApplicationContextUtils.getRequiredWebApplicationContext(sce.getServletContext());
		AnnotationConfigWebApplicationContext gwac = (AnnotationConfigWebApplicationContext) ac;
		gwac.close();
		log.debug("Web application destroyed");
	}

}